"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const prom_client_1 = require("prom-client");
const stream_1 = require("stream");
function createAutometricStreamPipe(name, baseOptions = {}) {
    const register = new prom_client_1.Registry();
    const metrics = {
        chunkSizes: new prom_client_1.Summary({
            help: `Autometric Summary for the chunk-size of the "${name}" Stream`,
            name: name + "_chunk_sizes",
            registers: [register],
        }),
        durations: new prom_client_1.Summary({
            help: `Autometric Summary for the durations of the "${name}" Promise`,
            name: name + "_durations",
            registers: [register],
        }),
        emits: new prom_client_1.Counter({
            help: `Autometric Counter for emitted Streams of "${name}"`,
            name: name + "_emits",
            registers: [register],
        }),
        ends: new prom_client_1.Counter({
            help: `Autometric Counter for ended Streams of "${name}"`,
            name: name + "_ends",
            registers: [register],
        }),
        incomingChunks: new prom_client_1.Counter({
            help: `Autometric Counter for incoming chunks of "${name}"`,
            name: name + "_incoming_chunks",
            registers: [register],
        }),
        nonEmits: new prom_client_1.Counter({
            help: `Autometric Counter for Streams of "${name}" without emitting data`,
            name: name + "_non_emits",
            registers: [register],
        }),
        numChunks: new prom_client_1.Summary({
            help: `Autometric Summary for the number of chunks of the "${name}" Promise`,
            name: name + "_num_chunks",
            registers: [register],
        }),
        throughput: new prom_client_1.Summary({
            help: `Autometric Summary for the throughput in bytes of the "${name}" Promise`,
            name: name + "_throughput",
            registers: [register],
        }),
    };
    return _a = class AutometricStreamPassThrough extends stream_1.PassThrough {
            constructor(options = {}) {
                options.labels = options.labels || {};
                options.rewriteLabels = options.rewriteLabels || ((passLabels) => passLabels);
                let currentLabels = Object.assign({}, AutometricStreamPassThrough.labels, options.labels);
                let firstChunk = false;
                let throughput = 0;
                let chunks = 0;
                let startDate = new Date(); // set already now a value to prevent bugs with non-emitting streams
                const transform = (chunk, encoding, callback) => {
                    chunks += 1;
                    currentLabels = options.rewriteLabels(currentLabels, chunk, encoding, baseOptions, options);
                    metrics.incomingChunks.inc(currentLabels, 1, new Date());
                    if (!firstChunk) {
                        firstChunk = true;
                        startDate = new Date();
                        metrics.emits.inc(currentLabels, 1, startDate);
                    }
                    if (chunk instanceof Buffer || typeof chunk === "string") {
                        metrics.chunkSizes.observe(currentLabels, chunk.length);
                        throughput += chunk.length;
                    }
                    callback(undefined, chunk);
                };
                super({
                    transform,
                });
                this.on("end", () => {
                    const endDate = new Date();
                    if (!firstChunk) {
                        metrics.nonEmits.inc(currentLabels, 1, endDate);
                    }
                    metrics.ends.inc(currentLabels, 1, endDate);
                    metrics.durations.observe(currentLabels, endDate.getTime() - startDate.getTime());
                    metrics.throughput.observe(currentLabels, throughput);
                    metrics.numChunks.observe(currentLabels, chunks);
                });
            }
        },
        _a.register = register,
        _a.prefix = name,
        _a.labels = baseOptions.labels || {},
        _a;
    var _a;
}
exports.createAutometricStreamPipe = createAutometricStreamPipe;
