"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const prom_client_1 = require("prom-client");
const stream_1 = require("stream");
const _1 = require("./");
class Pipe extends stream_1.PassThrough {
    constructor() {
        super({ transform: (chunk, encoding, next) => { next(undefined, chunk); } });
    }
}
describe("AutometricStream", () => {
    describe("creation of the Pass-Through class", () => {
        it("should return an instance of a Promise", () => {
            chai_1.expect(new (_1.createAutometricStreamPipe("instance_of_pass_through"))()).instanceOf(stream_1.PassThrough);
        });
        describe("static properties", () => {
            let AutometricStreamPipe;
            const name = "static_properties";
            const labels = { label: "just-a-test" };
            before(() => {
                AutometricStreamPipe = _1.createAutometricStreamPipe(name, {
                    labels,
                });
            });
            it("should have the prefix as prefix attribute", () => {
                chai_1.expect(AutometricStreamPipe.prefix).to.equal(name);
            });
            it("should have the labels as labels attribute", () => {
                chai_1.expect(AutometricStreamPipe.labels).to.deep.equal(labels);
            });
            it("should have the chunk_sizes summary in the register", () => {
                chai_1.expect(AutometricStreamPipe.register.getSingleMetric(name + "_chunk_sizes")).instanceOf(prom_client_1.Summary);
            });
            it("should have the durations summary in the register", () => {
                chai_1.expect(AutometricStreamPipe.register.getSingleMetric(name + "_durations")).instanceOf(prom_client_1.Summary);
            });
            it("should have the emits counter in the register", () => {
                chai_1.expect(AutometricStreamPipe.register.getSingleMetric(name + "_emits")).instanceOf(prom_client_1.Counter);
            });
            it("should have the ends counter in the register", () => {
                chai_1.expect(AutometricStreamPipe.register.getSingleMetric(name + "_ends")).instanceOf(prom_client_1.Counter);
            });
            it("should have the incoming chunks counter in the register", () => {
                chai_1.expect(AutometricStreamPipe.register.getSingleMetric(name + "_incoming_chunks")).instanceOf(prom_client_1.Counter);
            });
            it("should have the non_emits counter in the register", () => {
                chai_1.expect(AutometricStreamPipe.register.getSingleMetric(name + "_non_emits")).instanceOf(prom_client_1.Counter);
            });
            it("should have the num_chunks summary in the register", () => {
                chai_1.expect(AutometricStreamPipe.register.getSingleMetric(name + "_num_chunks")).instanceOf(prom_client_1.Summary);
            });
            it("should have the throughput summary in the register", () => {
                chai_1.expect(AutometricStreamPipe.register.getSingleMetric(name + "_throughput")).instanceOf(prom_client_1.Summary);
            });
        });
    });
    describe("work as a PassThrough stream", () => {
        let AutometricStreamPipe;
        const name = "as_stream";
        before(() => {
            AutometricStreamPipe = _1.createAutometricStreamPipe(name);
        });
        it("should pass data", (done) => {
            const inPipe = new Pipe();
            const outPipe = new Pipe();
            const autometricStream = new AutometricStreamPipe();
            inPipe.pipe(autometricStream).pipe(outPipe);
            outPipe.once("data", (data) => {
                chai_1.expect(data.toString()).equal("Just a test");
                done();
            });
            inPipe.write("Just a test");
        });
    });
    describe("counters", () => {
        let AutometricStreamPipe;
        const name = "counters";
        before(() => {
            AutometricStreamPipe = _1.createAutometricStreamPipe(name);
        });
        it("should count a emitted chunk", () => {
            AutometricStreamPipe.register.getSingleMetric(name + "_emits").reset();
            const inPipe = new Pipe();
            const outPipe = new Pipe();
            const autometricStream = new AutometricStreamPipe();
            inPipe.pipe(autometricStream).pipe(outPipe);
            outPipe.on("data", () => {
                // do nothing, just consume...
            });
            inPipe.write("Just a test");
            chai_1.expect(AutometricStreamPipe.register.getSingleMetric(name + "_emits")
                .get().values[0].value).equal(1);
        });
        it("should count an end", (done) => {
            AutometricStreamPipe.register.getSingleMetric(name + "_ends").reset();
            const inPipe = new Pipe();
            const outPipe = new Pipe();
            const autometricStream = new AutometricStreamPipe();
            inPipe.pipe(autometricStream).pipe(outPipe);
            outPipe.on("data", () => {
                // do nothing, just consume...
            });
            inPipe.end("Just a test");
            setTimeout(() => {
                chai_1.expect(AutometricStreamPipe.register.getSingleMetric(name + "_ends")
                    .get().values[0].value).equal(1);
                done();
            }, 10);
        });
        it("should count an end", (done) => {
            AutometricStreamPipe.register.getSingleMetric(name + "_incoming_chunks").reset();
            const inPipe = new Pipe();
            const outPipe = new Pipe();
            const autometricStream = new AutometricStreamPipe();
            inPipe.pipe(autometricStream).pipe(outPipe);
            outPipe.on("data", () => {
                // do nothing, just consume...
            });
            inPipe.write("first chunk");
            inPipe.write("second chunk");
            inPipe.end("third chunk");
            setTimeout(() => {
                chai_1.expect(AutometricStreamPipe.register.getSingleMetric(name + "_incoming_chunks")
                    .get().values[0].value).equal(3);
                done();
            }, 10);
        });
        it("should count a non-emit", (done) => {
            AutometricStreamPipe.register.getSingleMetric(name + "_non_emits").reset();
            const inPipe = new Pipe();
            const outPipe = new Pipe();
            const autometricStream = new AutometricStreamPipe();
            inPipe.pipe(autometricStream).pipe(outPipe);
            outPipe.on("data", () => {
                // do nothing, just consume...
            });
            inPipe.end();
            setTimeout(() => {
                chai_1.expect(AutometricStreamPipe.register.getSingleMetric(name + "_non_emits")
                    .get().values[0].value).equal(1);
                done();
            }, 10);
        });
    });
    describe("summary", () => {
        let AutometricStreamPipe;
        const name = "summaries";
        before(() => {
            AutometricStreamPipe = _1.createAutometricStreamPipe(name);
        });
        it("should measure execution duration", (done) => {
            AutometricStreamPipe.register.getSingleMetric(name + "_durations").reset();
            const inPipe = new Pipe();
            const outPipe = new Pipe();
            const autometricStream = new AutometricStreamPipe();
            inPipe.pipe(autometricStream).pipe(outPipe);
            outPipe.on("data", () => {
                // do nothing, just consume...
            });
            inPipe.write("test");
            setTimeout(() => {
                inPipe.end();
                setTimeout(() => {
                    const values = AutometricStreamPipe.register.getSingleMetric(name + "_durations")
                        .get().values;
                    chai_1.expect(values[values.length - 2].value).greaterThan(999).lessThan(1010);
                    done();
                }, 10);
            }, 1000);
        });
        it("should measure execution duration after the first chunk", (done) => {
            AutometricStreamPipe.register.getSingleMetric(name + "_durations").reset();
            const inPipe = new Pipe();
            const outPipe = new Pipe();
            const autometricStream = new AutometricStreamPipe();
            inPipe.pipe(autometricStream).pipe(outPipe);
            outPipe.on("data", () => {
                // do nothing, just consume...
            });
            setTimeout(() => {
                inPipe.write("test");
            }, 100);
            setTimeout(() => {
                inPipe.end();
                setTimeout(() => {
                    const values = AutometricStreamPipe.register.getSingleMetric(name + "_durations")
                        .get().values;
                    chai_1.expect(values[values.length - 2].value).greaterThan(890).lessThan(910);
                    done();
                }, 10);
            }, 1000);
        });
        it("should measure the number of chunks per stream", (done) => {
            AutometricStreamPipe.register.getSingleMetric(name + "_num_chunks").reset();
            const inPipe = new Pipe();
            const outPipe = new Pipe();
            const autometricStream = new AutometricStreamPipe();
            inPipe.pipe(autometricStream).pipe(outPipe);
            outPipe.on("data", () => {
                // do nothing, just consume...
            });
            let nums = 13;
            const intv = setInterval(() => {
                if (nums === 0) {
                    inPipe.end();
                    clearInterval(intv);
                    return;
                }
                nums -= 1;
                inPipe.write("chunk...");
            }, 10);
            outPipe.on("end", () => {
                const values = AutometricStreamPipe.register.getSingleMetric(name + "_num_chunks")
                    .get().values;
                chai_1.expect(values[values.length - 2].value).equal(13);
                done();
            });
        });
        it("should measure the throughput per stream", (done) => {
            AutometricStreamPipe.register.getSingleMetric(name + "_throughput").reset();
            const inPipe = new Pipe();
            const outPipe = new Pipe();
            const autometricStream = new AutometricStreamPipe();
            inPipe.pipe(autometricStream).pipe(outPipe);
            outPipe.on("data", () => {
                // do nothing, just consume...
            });
            for (let i = 0; i < 13; i += 1) {
                inPipe.write("chunk...");
            }
            inPipe.end();
            outPipe.on("end", () => {
                const values = AutometricStreamPipe.register.getSingleMetric(name + "_throughput")
                    .get().values;
                chai_1.expect(values[values.length - 2].value).equal(13 * 8);
                done();
            });
        });
    });
});
